@@ -27,6 +27,8 @@
  * See README
  */
 
+#define GNOME_PRINT_UNSTABLE_API
+
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomeprint/gnome-print-job.h>
 #include <libgnomeprintui/gnome-print-dialog.h>
@@ -60,8 +62,7 @@ my_print (void)
 	gpc    = gnome_print_job_get_context (job);
 
 	/* Run the dialog */
-	gtk_widget_show (dialog);
-	response = gtk_dialog_run (GTK_DIALOG (dialog));
+	response = gnome_print_dialog_run (GNOME_PRINT_DIALOG (dialog));
 	if (response == GNOME_PRINT_DIALOG_RESPONSE_CANCEL) {
 		g_print ("Printing was canceled\n");
 		return;
@@ -72,6 +73,9 @@ my_print (void)
 	my_draw (gpc);
 	gnome_print_job_close (job);
 	gnome_print_job_print (job);
+
+	g_object_unref (G_OBJECT (gpc));
+	g_object_unref (G_OBJECT (job));
 }
 
 int
